Sidebar = function() {
	this.mathxl_page = false;

	this.canvas;
	this.sidebarDiv;
	this.canvasDiv;

	this.title = null;
	this.first_problem = null;

	this.problems = {}; // problem text to textbox id
	this.expressions = {}; // problem textbox id to derivation ids
}

Sidebar.prototype.init = function() {
	console.log('using gmath version ', gmath.version);
	this.sidebarDiv = d3.select('body').append('div')
		.style({ 'width': '100%'
			     , 'height': '100%' });

	addCanvasDiv.call(this);
	startCanvas.call(this);

	this.canvasDiv.select('.gm-canvas').style('border', 'none')
	  .style('border-top', '1px solid silver');
	function addHeader() {
		// Add text input box, and button.
		var textDiv = this.canvasDiv.select('div.flex-container-columns')
		.insert('div', '*')
		.style({'padding': '10px'})
		.attr('id', 'textDiv');

		addLogo.call(this, textDiv);
		addInput.call(this, textDiv);
	}

	function addCanvasDiv() {
		var canvasWidth = this.sidebarDiv.node().scrollWidth;
		var canvasHeight = this.sidebarDiv.node().scrollHeight - 110;
		this.canvasDiv = this.sidebarDiv.append('div')
			.attr('id', 'canvasDiv')
			.style({ 'box-sizing': 'border-box'
				     , 'width': '100%'
				     , 'height': '100%'
				     , 'background': 'white'
				     });
	}

	function startCanvas() {
		adjustHelpEntries();
		gmath.setupLogging({ experiment_id: 'gm_sidebar', server: 'https://graspablemath.com:7010', enabled: false });
		gmath.TrialLogger.createTrial({ url: window.location.href });

		gmath.ui.resource_path = './';

		this.canvas = new gmath.Canvas(this.canvasDiv.node(), {
																									  'vertical_scroll': true
																									, 'insert_btn': true
																									, 'show_welcome_tutorial': true
																									, 'font_smaller_btn': false
																									, 'font_larger_btn': false
																									, 'btn_size':'xs'
																									, 'drawing': true
																									, 'gm_app_name': 'sidebar'
																									, 'saving_and_loading': true
																									, 'custom_login_method': login
																									, 'content_min_width': '300px'
																									, 'insert_menu_items': { derivation: true, function: true, textbox: true, video: true }
																									});
		this.canvas.model.fontSize(35);
		this.canvas.controller.use_hold_menu(false);

		this.canvas.model.on('create.gm-sidebar', (function(evt) {
					if (evt.type === 'create' && evt.target_type === 'derivation') {
						evt.target.row_events.on('dbltap', function (evt) { copyResult(evt.row) });
					}
			  }).bind(this));

		this.canvas.callWhenReady(this.initMessagePassing.bind(this));

		// this.resizeContentDiv();
		// window.addEventListener('resize', evt => this.updateSidebarWidth.bind(this));

		// canvas.showHint('Enter a math expression in the text field above or drag and drop math expressions from the left into the sidebar.', true);
	}

	function login() {
		window.top.postMessage(JSON.stringify({ sender: 'sidebar', login_request: true }), '*');
	}

	function addLogo(headerDiv) {
		var logo = headerDiv.append('a')
		  .style('float', 'left')
		  .style('margin-right', '15px')
		  .attr({ href: 'http://graspablemath.com/'
		 		    , target: '_blank' });
		logo.append('img')
		  .attr('src', chrome.extension.getURL("assets/gm-logo.png"))
		  .style({ height: '33px' });
	}

	function addInput(headerDiv) {
		var group = headerDiv.append('div')
		  .classed('input-group', true)
		  .style({ width: 'calc(100% - 55px)'
		  	     , float: 'left' });
		var self = this;
	  group.append('input')
	    .attr({class: 'form-control', type: 'text', placeholder: 'Enter math expression...'})
	    .on('keypress', function() {
					var key = d3.event.which || d3.event.keyCode;
					if (key !== 13) return;
					enteredExpr.call(self, this.value);
					this.value = '';
				});
	  group.append('span')
	    .classed('input-group-btn', true)
	    .append('button')
	    .attr({type: 'button', class: 'btn btn-default'})
	    .text('Create!')
	    .on('click', (function() {
				var input = group.select('input').node();
				enteredExpr.call(this, input.value);
				input.value = '';
			}).bind(this));
    headerDiv.append('div').style('clear', 'both');
	}

	function enteredExpr(enteredText) {
		if (!enteredText) { console.log('could not get data', data); return }
		math_str = enteredText.replace(/[.,\s\\]+$/, '');
		if (math_str.length === 0) return;
		try {
			var dl = this.canvas.model.createElement( 'derivation'
					                       , { eq: math_str
					                         , pos: {x: -1000, y: -1000} }
																 , 'toolbar', function(dl) {
																 	dl.canvas_model.moveToFreeSpace(dl);
																 } );
		} catch (err) {
			console.log('could not parse expression because', err);
		}
	}
}

Sidebar.prototype.addTextToCanvas = function(text, delta_y, scrollToShow, callback) {
	var canvasWidth = this.canvas.model.size().width;
	var textbox = this.canvas.model.createElement( 'textbox'
			                          , { text: text, pos: this.getLowestUnoccupiedPosition(delta_y)
			                            , size: { width: canvasWidth*0.9, height: 200 }
			                            , font_size: 24 }
													      , 'app'
													      , (function(el) {
													      													      		if (scrollToShow) this.canvas.model.scrollToElementTop(el, 40);
													      													      		if (callback) callback.call(this);
													      													        }).bind(this) );
	this.problems[text] = textbox.id;
	this.expressions[textbox.id] = [];
}

Sidebar.prototype.addDerivationToCanvas = function(expression, problem_number) {
	var dl = this.canvas.model.createElement('derivation'
					                       , { eq: expression, pos: 'auto' }
																 , 'app');
}

Sidebar.prototype.getLowestUnoccupiedPosition = function(delta_y) {
	var y = 0;
  this.canvas.model.elements().forEach(function(element) {
    var el_bottom = element.pos.y + element.size.height;
    if (el_bottom > y) y = el_bottom;
  });
  return { x: 20, y: y+delta_y };
}

Sidebar.prototype.findPositionForExpressionBelowProblem = function(problem_textbox, delta_y) {
	var bottom_of_textbox = problem_textbox.pos.y + problem_textbox.size.height;
	var expressions_for_problem = this.expressions[problem_textbox.id].map(function(derivation_id) {
		return this.canvas.model.getElementByID(derivation_id);
	}, this);
	var right_of_expressions = 0;
	for (var i=0; i<expressions_for_problem.length; i++) {
		var derivation = expressions_for_problem[i]
		  , right_side = derivation.pos.x + derivation.size.width;
		if (right_side > right_of_expressions) right_of_expressions = right_side;
	}
	return {x: right_of_expressions, y: bottom_of_textbox+delta_y};
}

function copyResult(row) {
	var match = row.model.match("\\VAR={answer:\\ANY}", "{answer:\\ANY}=\\VAR");
	var submission = match ? match.answer : row.model;
	// console.log(submission.to_latex());
	window.top.postMessage(JSON.stringify({ sender: 'sidebar', submission: submission.to_latex() }), '*');
}

Sidebar.prototype.startWorksheet = function() {
	if (!this.title || !this.first_problem) return;
	this.addTextToCanvas(this.first_problem, 20, false);
	this.canvas.user_data.problemset_name = this.title;
}

// [{mathxl_page}, {title}, {problem_number}, ...{problem_number}]
Sidebar.prototype.initMessagePassing = function() {
	window.addEventListener("message", receiveMessage.bind(this), false);

	function receiveMessage(event) {
		// console.log('received message', event);
		var parsed_data = $.parseJSON(event.data);
		if (parsed_data.sender === 'main') {
			if (parsed_data.gm_user) {
				window.gm_user = parsed_data.gm_user;
			}

			if (parsed_data.data) {
				for (var i=0; i<parsed_data.data.length; i++) {
					var datum = parsed_data.data[i];

					if (datum.mathxl_page) { // add submit button and remove share button
						this.canvas.addSubmitButton(true);
						var share_grp = d3.select('button#share').node();
						if (share_grp) share_grp.parentNode.remove();
						adjustHelpEntries(datum.mathxl_page);
					}

					if (datum.title) {
						this.title = datum.value;
						this.startWorksheet();
					}

					if (datum.problem_number) {
						this.first_problem = datum.value;
						this.startWorksheet();
					}

					if (datum.drop) {
						// console.log('calling handleDrop with', datum.drop);
						this.canvas.controller.handleDrop(datum.drop);
					}
				}
			}
		}

		else if (parsed_data.sender === 'document watcher') {
			if (parsed_data.expression) {
				this.addDerivationToCanvas(parsed_data.expression, parsed_data.problem_number);
			}

			else if (parsed_data.problem_number && this.problems[parsed_data.value]) {
				this.canvas.model.scrollToElementTop(this.canvas.model.getElementByID(this.problems[parsed_data.value]), 40);
			}

			else if (parsed_data.problem_number) {
				this.addTextToCanvas(parsed_data.value, Object.keys(this.problems).length === 0 ? 20 : 250, true);
			}
		}
	}

	window.top.postMessage(JSON.stringify({ sender: 'sidebar', ready: true }), '*');
}

function adjustHelpEntries(mathxl) {
	var entries = gmath.getHelpEntries();
	entries.forEach(function(entry) {
		if (entry.video) {
			if (entry.menu_title === "New! What's new in release 1.0?") {
				entry.menu_title = "Watch the how-to video!";
				entry.modal_title = "Graspable Math Sidebar How-to Video";
				entry.video = "VqD-wMrLvb8";
			}
			entry.external_page = true;
			entry.src = 'https://youtu.be/' + entry.video;
			entry.icon = "glyphicon glyphicon-new-window";
			delete entry.video;
		}
		else if (entry.src && entry.src.indexOf('introduction.html') !== -1) {
			if (mathxl) {
				entry.src = "/shared/content/help-pages/introduction-mathxl.html";
				entry.parts = 6;
			}	else {
				entry.src = "/shared/content/help-pages/introduction-sidebar.html";
			}
		}
	});
}

sidebar = new Sidebar();
sidebar.init();
