var gm_user = null;
var DISABLE_MATHXL = true;
// flow:
// 1) browser action clicked/tab update completion on mathxl page => message sent to content script
// 2) content script responds asking for libraries
// 3) send message saying libraries executed
// 4) content script waits for libraries to be defined, sends message asking for content scripts
// 5) send message saying content scripts executed
// 6) content script waits for content scripts to be defined, sends message asking for css
// 7) send message saying css is inserted
// 8) content script starts sidebar
chrome.browserAction.onClicked.addListener(function(tab) {
	if (!displayingMathXLPage(tab)) {
		login(false, tab);
	}

	chrome.tabs.sendMessage( tab.id
		                     , { sender: 'inject'
		                     	 , browser_action: true
		                     	 , mathxl_page: displayingMathXLPage(tab) }
		                     , function(response) {
		if (response && response.inject_libraries) injectLibraries(tab);
	})
});

function getURLParameters(search) {
	var params = {};
	search.replace(/([^&=]+)=([^&=]+)/g, function(all, $1, $2) {
		params[$1] = decodeURIComponent($2);
	});
	return params;
}

function login(interactive, tab) {
	getGoogleOAuth2Token(interactive, function(err, data) {
		if (err || !data || !data.access_token) return;
		authorizeGMUser(data.access_token, tab);
	});
}

function getGoogleOAuth2Token(interactive, cb) {
	var clientId_gm_web = '81925747312-vao5h4l5bf2c8cbfssff1bejaut2at9n.apps.googleusercontent.com';
	var scopes = "email profile";

	var authUrl = 'https://accounts.google.com/o/oauth2/v2/auth'
      + '?response_type=token&client_id=' + clientId_gm_web
      + '&scope=' + scopes
      + '&redirect_uri=' + chrome.identity.getRedirectURL("oauth2");

	console.log('running launchWebAuthFlow with url', authUrl);

	chrome.identity.launchWebAuthFlow({'url': authUrl, 'interactive': interactive}, function(redirect_url) {
	  if (redirect_url) {
	  	console.log('Success!', redirect_url);
	  	let idx = redirect_url.indexOf('#');
	  	let params = getURLParameters(redirect_url.substr(idx+1));
	  	cb(null, params);
	  }
    else {
    	console.log('Failed! Have you registered this redirect url?', chrome.identity.getRedirectURL("oauth2"));
    	cb(true);
    }
  });
}

function authorizeGMUser(access_token, tab) {
	console.log('Authorizing GM user with access token', access_token);
	var xhr = new XMLHttpRequest();
	xhr.onreadystatechange = function() {
		if (this.readyState === 4) {
			if (this.status === 200) {
				gm_user = JSON.parse(this.responseText);
				console.log('got user back from gm server!', gm_user);
				if (tab) sendUser(tab, gm_user);
			} else {
				console.warn(this.status, this.responseText);
			}
		}
	}
	xhr.open('GET', "https://graspablemath.com/auth/google/token?access_token="+access_token);
	xhr.setRequestHeader('Content-Type', 'application/json');
	xhr.send();
}

function sendUser(tab, gm_user) {
	chrome.tabs.sendMessage( tab.id
   , { sender: 'inject'
     , user: gm_user });
}

chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab) {
	// console.log(changeInfo);
	if (changeInfo.status === 'complete' && displayingMathXLPage(tab)) {
		chrome.tabs.sendMessage( tab.id
			                     , { sender: 'inject'
			                     	 , tab_update: true
			                     	 , mathxl_page: displayingMathXLPage(tab) }
			                     , function(response) {
			// console.log(response);
			if (response.inject_libraries) injectLibraries(tab);
		})
	}
})

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
	// console.log('received message in inject script', request, sender);
	if (request.sender === 'main' && request.script_requests) {
		// console.log('tab', sender.tab.id, 'requested', request.script_requests);
		executeScripts(sender.tab, request.script_requests, function() {
			chrome.tabs.sendMessage(sender.tab.id, { sender: 'inject', scripts_injected: true }, function(response) {
				// console.log(response);
			})
		});
	}

	else if (request.css_requests) {
		// console.log('tab', sender.tab.id, 'requested', request.css_requests);
		insertCSSList(sender.tab, request.css_requests, { allFrames: false }, function() {
			chrome.tabs.sendMessage(sender.tab.id, { sender: 'inject', css_injected: true }, function(response) {
				// console.log(response);
			})
		});
	}

	else if (request.login_request) {
		login(true, sender.tab);
	}

	else {
		// console.log('tab', sender.tab.id, 'requested that', request, 'be broadcast');
		sendResponse({ message_received: true });
		chrome.tabs.sendMessage(sender.tab.id, request, function(response) {
			// console.log(response);
		});
		return;
	}

	sendResponse({ message_received: true });
});

function displayingMathXLPage(tab) {
	if (DISABLE_MATHXL) return false;
	return tab.url && tab.url.indexOf('https://www.mathxlforschool.com/Student/PlayerHomework.aspx') !== -1;
}

function injectLibraries(tab) {
	var scripts = []
	  , css_list = [];
	scripts.push({ file: "assets/d3.min.js" });
	scripts.push({ file: "assets/jquery-3.1.0.min.js" });
	executeScripts(tab, scripts, function() {
		chrome.tabs.sendMessage(tab.id, { sender: 'inject', libraries_injected: true }, function(response) {
		})
	});
}

function executeScripts(tab, script_info, callback) {
	if (script_info.length === 0) callback();
	for (var i=0; i<script_info.length; i++) {
		chrome.tabs.executeScript( tab.id
			                       , script_info[i]
			                       , i < script_info.length-1 ? null : callback );
	}
}

function insertCSSList(tab, css_list, options, callback) {
	options = options || {};
	for (var i=0; i<css_list.length; i++) {
		chrome.tabs.insertCSS( tab.id
			                   , Object.assign({ file: css_list[i] }, options)
			                   , i < css_list.length-1 ? null : callback );
	}
}
