var gm_user = null;

var mathxl_page = false
  , mathxl_page_ready = false
  , libraries_injected = false
  , sidebar_injected = false
  , css_injected = false
  , application_started = false;

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
	if (request.sender === 'inject' && request.user) {
		gm_user = request.user;
		if (canvas_is_ready) {
			iframe_node.contentWindow.postMessage(JSON.stringify({ sender: 'main', gm_user: gm_user }), '*');
		}
		sendResponse({ message_received: true });

	}

	else if (request.sender === 'inject' && !application_started) {
		// determined by the event page based on the URL of the browser action's tab
		if (request.mathxl_page) {
			mathxl_page = request.mathxl_page;
			queued_data.push({mathxl_page: mathxl_page});
		}

		if (request.libraries_injected) {
			libraries_injected = request.libraries_injected;
			// We don't want to execute (i.e. run) the document watchers until we know the mathxl page
			// is loaded and contains the content we wish to observe.
			waitForLibraryLoad(mathxl_page ?
				                 waitForMathXLPageLoad.bind(null, requestContentScripts.bind(null, mathxl_page)) :
				                 requestContentScripts.bind(null, mathxl_page));
		}

		if (request.scripts_injected) {
			sidebar_injected = true;
			waitForContentScriptLoad(mathxl_page, requestCSS.bind(null, mathxl_page));
		}

		if (request.css_injected) {
			css_injected = true;
			openSidebar(mathxl_page);
		}

		sendResponse({ mathxl_page: mathxl_page
			           , inject_libraries: !libraries_injected
		             , sidebar_injected: sidebar_injected
		             , css_injected: css_injected });
	}

	else if (request.sender === 'inject' && application_started && opened) {
		closeSidebar(request.mathxl_page);
	}

	else if (request.sender === 'inject' && application_started && !opened) {
		queued_data.push({mathxl_page: request.mathxl_page});
		openSidebar(mathxl_page);
	}

	else if (request.sender === 'document watcher') {
		// console.log('piping request from document watcher', request);
		if (!canvas_is_ready) queued_data.push(request);
		else iframe_node.contentWindow.postMessage(JSON.stringify(request), '*');
	}
});

// Listen for messages posted on the DOM from the mathxl page and our iframe.
window.addEventListener("message", receiveMessage, false);

function waitForLibraryLoad(callback) {
	var libCheck = setInterval(function() {
		if (typeof(d3) !== 'undefined' &&
				typeof(jQuery) !== 'undefined') {
			clearInterval(libCheck);
			if (callback) callback();
		}
	}, 10);
}

function waitForContentScriptLoad(mathxl_page, callback) {
	var scriptCheck = setInterval(function() {
		if (!mathxl_page ||
			  typeof(PearsonParser) !== 'undefined' && typeof(documentWatcher) !== 'undefined') {
			clearInterval(scriptCheck);
			if (callback) callback();
		}
	}, 10)
}

// We have a 4-second wait-limit here, to cover for the cases where their message system is not
// working as we've observed during development, or if perhaps their page is loaded before we have
// a chance to listen to the completion message.
function waitForMathXLPageLoad(callback) {
	var limit = 4000;
	var pageCheck = setInterval(function() {
		if (mathxl_page_ready || limit <= 0) {
			clearInterval(pageCheck);
			if (callback) callback();
		}
		limit -= 10;
	}, 10)
}

function requestContentScripts(mathxl_page) {
	var scripts = [];
	if (mathxl_page) {
		scripts.push({ file: "scripts/pearson-html-parser.js", allFrames: true });
		scripts.push({ file: "scripts/document-watcher.js", allFrames: true });
	}

	chrome.runtime.sendMessage({ sender: 'main', script_requests: scripts }, function(response) {
		// console.log(response);
	});
}

function requestCSS(mathxl_page) {
	chrome.runtime.sendMessage( { sender: 'main', css_requests: [ mathxl_page
																						                  ? "assets/mathxl-sidebar-styling.css"
																						                  : "assets/sidebar-styling.css"] }
	                          , function(response) {
	  // console.log(response);
  });
}

var queued_data = [];
var opened = false
  , resize_div = null
  , sidebar_div = null
  , iframe_node = null
  , dropTarget = null
  , event_block_div = null
  , canvas_is_ready = false
  , sidebar_visible = false
  , rs_toggle = null
  , rs_inner = null;

function openSidebar(mathxl_page) {
	application_started = true;
	d3.select('body').classed('flex-container', true);

	if (!mathxl_page) {
		$("body").children().wrapAll("<div id='originalPage'></div>");
		d3.select('#originalPage').on('dragstart', function() {
			// this allows drag-and-drop to work
			d3.select(iframe_node).style('pointer-events', 'none');
		}).on('dragend', function() {
			d3.select(iframe_node).style('pointer-events', null);
		});
	}

	initResizeBar();
	initSidebar();
	// percent into px, otherwise the width: inherit of the iframe is buggy
	sidebar_div.style('width', sidebar_div.node().clientWidth + 'px');
	setSidebarVisibility(mathxl_page ? false : true);
	opened = true;
}

function createDropTarget() {
	dropTarget = sidebar_div.append('div')
	  .style('position', 'absolute')
	  .style('width', '100%')
	  .style('height', '100%')
	  .style('top', '0')
	  .style('left', 0);
	dropTarget.append('div')
	  .style({ position: 'fixed'
	         , width: 'inherit'
	         , height: '100%' })
	  .style('background', '#dff0d8')
	  .style('opacity', 0.5)
	  .on('dragover', function() { d3.event.preventDefault() })
	  .on('drop', triggerDrop);
}

function triggerDrop() {
	// console.log('trigger Drop');
	var math_str = getImageAttr(d3.event.dataTransfer, 'alt') || getMathFromText(d3.event.dataTransfer);
	var img_url = getImageAttr(d3.event.dataTransfer, 'src');
	if (!math_str) return;
	var container = iframe_node;
  var rect = container.getBoundingClientRect();
	var data = {
		clientX: d3.event.clientX - rect.left - container.clientLeft
	, clientY: d3.event.clientY - container.clientTop // fixed position
	, math_str: math_str
	, img_url: img_url
	}
	iframe_node.contentWindow.postMessage(JSON.stringify({ sender: 'main', data: [{drop: data }]}), '*');
	// console.log('drop event', data);
}

function getImageAttr(dataTransfer, attr) {
  var data = dataTransfer.getData("text/html");
  var tmp = document.createElement('div');
  var img = d3.select(tmp).html(data).select('img');
  return img.size() === 1 ? img.attr(attr) : null;
}

function getMathFromText(dataTransfer) {
  return dataTransfer.getData('text/plain');
}

function closeSidebar(mathxl_page) {
	window.removeEventListener("message", receiveMessage);
	d3.select("#resizeBar").remove();
	d3.select("#wikiBar").remove();

	if (!mathxl_page) {
		$("#originalPage").replaceWith(function() { return $(this).html() });
	}

	canvas_is_ready = false;
	opened = false;
	d3.select('body').classed('flex-container', false);
}

function setSidebarVisibility(vis) {
	sidebar_visible = vis;
	sidebar_div.style('display', sidebar_visible ? null : 'none');
	resize_div.style('width', sidebar_visible ? '10px' : '0');
	rs_toggle.text(sidebar_visible ? '▶' : '◀');
	rs_toggle.style('text-align', sidebar_visible ? 'right' : 'center')
}

function initResizeBar() {
	resize_div = d3.select('body').append('div')
		.attr('id', 'resizeBar');

	rs_inner = resize_div.append('div').attr('id', 'gm-rs-inner');

	rs_toggle = resize_div.append('div').attr('id', 'gm-rs-toggle');
	rs_toggle.text('◀')
	  .on('click', function() {
	  	setSidebarVisibility(!sidebar_visible)
	  })
	  .on('mousedown', function() { d3.event.stopPropagation() })
	  .on('touchstart', function() { d3.event.stopPropagation() });
	//◀▶◁▷

	var dragResize = d3.behavior.drag()
	  .on('dragstart', function() {
	  	if (!sidebar_visible) return;
	  	if (event_block_div) event_block_div.style('display', null);
		})
		.on('drag', (function() {
			if (!sidebar_visible) return;
			var new_width = Math.max(335, sidebar_div.node().clientWidth-d3.event.dx);
			sidebar_div.style('width', new_width + 'px');
		}).bind(this))
		.on('dragend', function() {
			if (!sidebar_visible) return;
			if (event_block_div) event_block_div.style('display', 'none');
		});
	resize_div.call(dragResize);
}

function initSidebar() {
	sidebar_div = d3.select('body').append('div')
		.attr('id', 'wikiBar')

	createDropTarget();
	
	iframe_node = sidebar_div.append('iframe')
		.attr('id', 'wikiBarIFrame')
 		.attr('src', chrome.extension.getURL("assets/graspable-math/sidebar.html"))
 		.node();

	event_block_div = d3.select('body').append('div')
		.style({ 'position': 'fixed'
		       , 'width': '100%'
		       , 'height': '100%'
		       , 'z-index': 1000
		       , background: 'transparent'
		       , display: 'none' });
}

function receiveMessage(event) {
	// ignore any messages not from our own extension or the mathxl page
	if (event.origin !== chrome.extension.getURL('').slice(0,-1) && event.origin !== "https://xlitemprod.pearsoncmg.com") return;

	// We are listening to messages on the document starting when the tab is created, so the libraries
	// may not be loaded yet if we receive a message from the mathxl app.
	if (typeof(jQuery) === 'undefined') return;

	var parsed_data;
	if (typeof(event.data) === 'string') var parsed_data = $.parseJSON(event.data);
	else parsed_data = event.data;
	// console.log('received message in the main content script:', parsed_data);

	// Message from the mathxl page.  Seems to signify that all their frames are loaded.  This is when
	// we can search the page for information relevant to us.
	if (parsed_data['@type'] && parsed_data['status'] === 'OK') {
		mathxl_page_ready = true;
	}

	// The following messages are from our sidebar iframe.
	else if (parsed_data.sender === 'sidebar' && !canvas_is_ready) {
		canvas_is_ready = true;
		iframe_node.contentWindow.postMessage(JSON.stringify({ sender: 'main', data: queued_data, gm_user: gm_user }), '*');
		if (queued_data.length > 2) queued_data = queued_data.slice(0, 2);
	}

	else if (parsed_data.sender === 'sidebar' && parsed_data.login_request) {
		// The "inject" background script will send another message once the login process has completed.
		chrome.runtime.sendMessage(parsed_data, function(response) {
			// console.log(response)
		});
	}

	else if (parsed_data.sender === 'sidebar') {
		chrome.runtime.sendMessage(parsed_data, function(response) {
			// console.log(response);
		});
	}
}

