// str = '<span class="eqDocument" aria-hidden="true" role="presentation"><span class="eqCBrace"><span class="left eqParenSvg" style="font-size: inherit; height: 35px; margin-top: 2px;"><svg height="35.5" width="6.5"><path d="M5.5,0.5 A 4,8 0 0 0 1.5,8.5 L1.5 27 A 4,8 0 0 0 5.5 34.5" style="fill:none;stroke:#000000; stroke-width:1"></path></svg></span><span class="eqRow"><span class="eqDivide"><span class="eqRow eqNumerator"><span class="txtNum"><span class="">4</span></span></span><span class="eqRow eqDenominator"><span class="txtNum"><span class="">7</span></span></span><span style="display: inline-block; width: 0px;"></span></span></span><span class="right eqParenSvg" style="font-size: inherit; height: 35px; margin-top: 2px;"><svg height="35.5" width="6.5"><path d="M0.5,0.5 A 4,8 0 0 1 4.5,8.5 L4.5 27 A 4,8 0 0 1 0.5 34.5" style="fill:none;stroke:#000000; stroke-width:1"></path></svg></span></span><span class="eqSuperscript" style="vertical-align: 15.15px;"><span class="eqRow eqSupContent"><span class="txtNum"><span class="">2</span></span></span></span></span>';

PearsonParser = function() {
	this.class_names = {}; // class_name => function
}

PearsonParser.latexToInputCommands = function(latex) {
	var transformed_latex = latex
		.replace(/\^([^\{])/g, function(match, exp_val) { return "^{" + exp_val + "}" })
		.replace(/\\cdot/g, " • ")
		.replace(/\\frac/g, "/ ")
		.replace(/{/g, "")
		.replace(/}/g, " ; ") // escape from a group
		.replace(/\\left\(/g, " (")
		.replace(/\\right\)/g, ") ") // simple brackets do not require escaping
		.replace(/\\left\[/g, " [")
		.replace(/\\right\]/g, "] ")
		.replace(/\\left\|/g, "\\abs ") // \abs will fulfill the app's internal model, "|"" will not
		.replace(/\\right\|/g, " ; ") // absolute value groups require escaping
		.replace(/\\sqrt\[(.*?)\]/, function(match, degree) { return "\\nrt " + degree + " ; " });

	var commands = transformed_latex
		.split(" ")
		.filter(function(str) { return str.length > 0 })
		.map(function(str) {
			if (str === ';') return { key: 'right' };
			else if (str === '•') return { key: 'dot' }
			else return { input: str };
		});

	while (commands.find(function(command) { return command.input && command.input === '(/' })) {
		var open_idx = commands.findIndex(function(command) { return command.input && command.input === '(/' });
		var close_idx = commands.findIndex(function(command, close_idx) { return close_idx > open_idx && command.input && command.input === ')' });
		commands.splice(close_idx, 1, {key: 'right'});
		commands.splice(open_idx, 1, {input: '('}, {input: ')'}, {key: 'left'}, {input: '/'});
	}

	return commands;
}

// If we allow any of the Pearson HTML-styled expressions on the page to be parsed and put in the
// sidebar (which is useful if side-expressions are included in the problem description), then
// we often get along with them punctuation, which for whatever reason is included with the math
// expression. Strip the trailing punctuation as a final parsing step. "=" is included so the user
// can also click on the expression next to the input field (which is like x/x = <input field>).
// We interpret the input field as an empty string.
PearsonParser.prototype.parsePearsonExpression = function(node) {
	var parsed_str = this.parseNode(node);
	while ([',', '=', '.', ' ', '"', '\''].indexOf(parsed_str[parsed_str.length-1]) !== -1) parsed_str = parsed_str.slice(0, -1);
	parsed_str = this.replaceBlanksWithNewVariable(this.normalizeSymbols(parsed_str));
	return parsed_str;
}

PearsonParser.prototype.normalizeSymbols = function(str) {
	return str
		.replace(/[\"\'\$]/g, '')
		.replace(/−/g, '-')
		.replace(/•/g, '*');
}

// Intended to cover: <span class="eqDocument" aria-hidden="true" role="presentation"><span class="eqDivide"><span class="eqRow eqNumerator"><span class="txtNum"><span class="">3</span></span></span><span class="eqRow eqDenominator"><span class="txtNum"><span class="">4</span></span></span><span style="display: inline-block; width: 0px;"></span></span><span class="eqOp"><span class="">=</span></span><span class="eqDivide"><span class="eqRow eqNumerator"><span><span class="eqSpace">&nbsp;</span></span></span><span class="eqRow eqDenominator"><span class="txtNum"><span class="">2</span><span class="">0</span></span></span><span style="display: inline-block; width: 0px;"></span></span></span>
// and: <span class="eqLabel fNormal" id="xl_eqeditor_EqLabel_12" widgetid="xl_eqeditor_EqLabel_12" aria-label="three fourths equals StartFraction nothing Over 20 EndFraction" style="background-color: transparent !important;"><span class="eqAccessibleLabel">three fourths equals StartFraction nothing Over 20 EndFraction</span><span class="eqDocument" aria-hidden="true" role="presentation"><span class="eqDivide"><span class="eqRow eqNumerator"><span class="txtNum"><span class="">3</span></span></span><span class="eqRow eqDenominator"><span class="txtNum"><span class="">4</span></span></span><span style="display: inline-block; width: 0px;"></span></span><span class="eqOp"><span class="">=</span></span><span class="eqDivide"><span class="eqRow eqNumerator"><span class="eqObject" style="font-size: 13px;"><span class="eqEditor inputField" widgetid="S1"><span class="hiddenInputContainer"><input type="text" class="focusNode" autocapitalize="off" autocomplete="off" autocorrect="off" spellcheck="false" tabindex="0" id="S1"></span><span class="eqDocument empty current" aria-hidden="true" role="presentation"></span><span class="eqAccessibleLabel"></span></span></span></span><span class="eqRow eqDenominator"><span class="txtNum"><span class="">2</span><span class="">0</span></span></span><span style="display: inline-block; width: 0px;"></span></span></span></span>
// The first is 3/4=<blank>/20, the second is the same, but with an input field where the blank is.
// This is a niche case for fractions, because normally a blank in any other expression probably
// can't convey a math problem very well.  The second HTML example here is even more niche, as we
// have added {} to encapsulate the contents of the fraction's numerator, although the term in the
// numerator was interpreted as the empty string.
PearsonParser.prototype.replaceBlanksWithNewVariable = function(str) {
	return str.replace(/&nbsp;|{}/g, '"?"');
}

PearsonParser.prototype.parseNode = function(node) {
	var class_name = node.classList[0];
	// console.log('class_name name is:', class_name);
	if (class_name && !(class_name in this.class_names)) throw "Unknown class name " + class_name;
	var res = this.class_names[class_name || 'NONE'](node);
	// console.log('transformed into', res);
	return res;
}

/// Can pass array of or single node.
PearsonParser.prototype.parseNodes = function(nodes) {
	var res = [];
	var wrap_next_term_with_curlies = false;
	for (var i = 0; i < nodes.length; i++) {
		var parsed_node = this.parseNode(nodes[i]);
		if (parsed_node === '÷') {
			parsed_node = '*1/';
			wrap_next_term_with_curlies = true;
		} else if (wrap_next_term_with_curlies) {
			parsed_node = '{' + parsed_node + '}';
			wrap_next_term_with_curlies = false;
		}
		res.push(parsed_node);
	}
	return res.join('');
}

PearsonParser.prototype.registerClassName = function(class_name, handler) {
	this.class_names[class_name] = handler;
}

PearsonParser.prototype.registerClassNames = function(class_names, handler) {
	for (var i = 0; i < class_names.length; i++) this.registerClassName(class_names[i], handler);
}

pearsonParser = new PearsonParser();

pearsonParser.registerClassName('eqLabel', function(node) {
	return pearsonParser.parseNode(node.children[1]);
});

// This element contains the user input field.
pearsonParser.registerClassNames(['eqEditor', 'eqObject'], function(node) {
	return '';
});

// I have seen two kinds of expression rendering styles in the Pearson homework app so far.
// If the expression is rendered within an eqDocument span, then the terms inside are defined using
// spans with tags that describe its structure.
// If the expression is rendered within a nowrap div, then the terms inside are each wrapped with
// an eqLabel classed span, which is the type of element that would normally wrap an entire eqDocument.
pearsonParser.registerClassNames(['nowrap', 'eqDocument', 'eqRow'], function(node) {
	return pearsonParser.parseNodes(node.children);
});

// SVG paren.
pearsonParser.registerClassName('eqCBrace', function(node) {
	return "(" + pearsonParser.parseNode(node.children[1]) + ")";
});

// Non-SVG paren.
pearsonParser.registerClassName('eqParen', function(node) {
	if (node.classList[1] === "left") return "("; // ex: <span class="eqParen left eqParenSvg" style="font-size: inherit; height: 35px; margin-top: 5px;"><svg height="35.5" width="6.5"><path d="M5.5,0.5 A 4,8 0 0 0 1.5,8.5 L1.5 27 A 4,8 0 0 0 5.5 34.5" style="fill:none;stroke:#000000; stroke-width:1"></path></svg></span>
	else if (node.classList[1] === "right") return ")";
	else return node.innerHTML;
});

// fraction
pearsonParser.registerClassName('eqDivide', function(node) {
	return "{" + pearsonParser.parseNodes(node.children[0].children) +
				 "}/{" + pearsonParser.parseNodes(node.children[1].children) + "}";
});

pearsonParser.registerClassName('eqSuperscript', function(node) {
	return "^" + pearsonParser.parseNode(node.children[0].children[0]);
});

pearsonParser.registerClassName('eqAbs', function(node) {
	return "|" + node.children[0].children[0] + "|"; // ex: <span class="eqAbs" style="margin-left: 2px; border-left: 1px solid; margin-right: 2px; border-right: 1px solid;"><span class="eqRow eqAbsRow current" style="margin-left: 2px; margin-right: 2px;"><span><span class="">s</span></span></span></span>
});

pearsonParser.registerClassName('eqSqrt', function(node) {
	return "sqrt{" + node.children[0].children[1] + "}"; // ex: <span class="eqSqrt"><span class="radicand-holder"><span class="radical-symbol"><svg height="18" width="18.05"><path style="none" stroke="#000000" stroke-width="1" fill="none" d="M0.5,11.5 l2,-2 L4.5,16.5 L8.5,2.5 L18.5,2.5"></path></svg></span><span class="eqRow eqRadicand current"><span><span class="">a</span></span></span></span></span>
});

pearsonParser.registerClassName('opPlus', function(node) {
	return '+';
});

// Variables often have no class.  Handle that here as "NONE."
pearsonParser.registerClassNames(['NONE', 'eqOp'], function(node) {
	return node.children[0].innerHTML;
});

// Each number token has its own span (i.e. the number "10" is rendered w/ two spans).
pearsonParser.registerClassName('txtNum', function(node) {
	var num_str = "";
	for (var i=0; i<node.children.length; i++) {
		num_str += node.children[i].innerHTML;
	}
	return num_str;
});

// This element contains random math tokens.  See the example below, which contains "[(".
pearsonParser.registerClassName('rvTxt', function(node) {
	if (node.children.length > 0) return node.children[1].innerHTML; // ex: <span class="rvTxt fNormal"><span class="wbr">&#8203;</span><span class="nowrap">[(</span></span>
	return node.innerHTML;
});
